@@ -6,9 +6,11 @@ License: MIT
 Homepage: https://github.com/KiTant/CTkCodeBoxPlus
 """
 
-__version__ = '1.0'
+__version__ = '1.2'
 
 from .ctk_code_box import CTkCodeBox
+from .dataclasses import *
+from .custom_exception_classes import *
 from .text_menu import TextMenu
 from .add_line_nums import AddLineNums
 from .keybinding import _unregister_keybind, _register_keybind
