@@ -9,36 +9,15 @@ import customtkinter
 from .text_menu import TextMenu
 from .add_line_nums import AddLineNums
 from pygments import lex, lexer
-from pygments.lexers import (python, javascript, c_cpp, dotnet, html,
-                             css, data, go, jvm, php, ruby, rust,
-                             scripting, perl, objective, jsx)
 from pygments.styles import get_style_by_name, get_all_styles
+from .constants import common_langs
+from .custom_exception_classes import *
+from .dataclasses import *
 from .keybinding import _register_keybind
 from typing import Union
 import pyperclip
 
 
-# Custom Exception Classes
-class CTkCodeBoxError(Exception):
-    """Base exception for CTkCodeBoxPlus errors."""
-    pass
-
-
-class LanguageNotAvailableError(CTkCodeBoxError):
-    """Raised when a requested language is not available or not recognized."""
-    pass
-
-
-class ThemeNotAvailableError(CTkCodeBoxError):
-    """Raised when a requested theme name is invalid or not installed."""
-    pass
-
-
-class LexerError(CTkCodeBoxError):
-    """Raised when a provided lexer is invalid or fails to tokenize input."""
-    pass
-
-
 class CTkCodeBox(customtkinter.CTkTextbox):
     """A code-oriented CTk textbox with syntax highlighting and UX helpers.
 
@@ -58,20 +37,14 @@ class CTkCodeBox(customtkinter.CTkTextbox):
                  language: Union[str, lexer.Lexer],
                  height: int = 200,
                  theme: str = "solarized-light",
-                 line_numbering: bool = True,
-                 numbering_color: str = None,
-                 menu: bool = True,
-                 menu_fg_color: str = None,
-                 menu_text_color: str = None,
-                 menu_hover_color: str = None,
+                 numbering_settings: NumberingSettings = NumberingSettings(),
+                 menu_settings: MenuSettings = MenuSettings(),
                  wrap: bool = True,
                  select_color: str = None,
                  cursor_color: str = None,
                  highlight_current_line: bool = True,
                  highlight_current_line_color: str = None,
-                 history_enabled: bool = True,
-                 history_cooldown: int = 2500,
-                 history_max: int = 100,
+                 history_settings: HistorySettings = HistorySettings(),
                  indent_width: int = 4,
                  **kwargs):
         """Initialize a CTkCodeBox instance.
@@ -81,25 +54,19 @@ class CTkCodeBox(customtkinter.CTkTextbox):
             language: Pygments language name (str) or a lexer class.
             height: Widget height in pixels (passed to CTkTextbox).
             theme: Pygments style name used for highlighting.
-            line_numbering: Enable line numbers.
-            numbering_color: Color for line numbers.
-            menu: Enable context menu.
-            menu_fg_color: Context menu background color.
-            menu_text_color: Context menu text color.
-            menu_hover_color: Context menu active background color.
+            numbering_settings: NumberingSettings object for the line nums.
+            menu_settings: MenuSettings object for the context menu.
             wrap: Enable word wrap.
             select_color: Override selection background color.
             cursor_color: Cursor color I (blinking).
             highlight_current_line: Highlight the active line.
             highlight_current_line_color: Explicit color for active line.
-            history_enabled: Enable built-in undo/redo history.
-            history_cooldown: Cooldown for pushing history in _on_keypress_history().
-            history_max: Maximum undo frames to keep.
+            history_settings: HistorySettings object for custom history.
             indent_width: Number of spaces for indent/outdent.
             **kwargs: Additional arguments passed to CTkTextbox.
         """
         # Do not enable Tk's built-in undo/history
-        if 'undo' in kwargs and history_enabled:
+        if 'undo' in kwargs and history_settings.enabled:
             try:
                 del kwargs['undo']
             except Exception:
@@ -109,8 +76,11 @@ class CTkCodeBox(customtkinter.CTkTextbox):
         # Wrap management
         self.wrap_enabled = bool(wrap)
         self.set_wrap(self.wrap_enabled)
-        if line_numbering:
-            self.line_nums = AddLineNums(self, text_color=numbering_color)
+        if numbering_settings.enabled:
+            self.line_nums = AddLineNums(self, text_color=numbering_settings.color,
+                                         justify=numbering_settings.justify,
+                                         padx=numbering_settings.padx,
+                                         auto_padx=numbering_settings.auto_padx)
 
         self.select_color = select_color
         if select_color:
@@ -133,12 +103,10 @@ class CTkCodeBox(customtkinter.CTkTextbox):
         _register_keybind(self, "CmdOrCtrl+Z", lambda: self.undo(), bind_scope='widget')
 
         # Custom history (undo/redo)
-        self._history_enabled = history_enabled
-        self._history_max = history_max
         self._undo_stack = []
         self._redo_stack = []
         self._history_typing_cooldown = None
-        self._history_cooldown = history_cooldown
+        self.history_settings = history_settings
 
         # Job id for debounced highlighting
         self._highlight_job = None
@@ -172,30 +140,6 @@ class CTkCodeBox(customtkinter.CTkTextbox):
         self.theme_name = theme
         self.all_themes = list(get_all_styles())
 
-        self.common_langs = {
-            "python": python.PythonLexer,
-            "c": c_cpp.CLexer,
-            "cpp": c_cpp.CppLexer,
-            "c++": c_cpp.CppLexer,
-            "c#": dotnet.CSharpLexer,
-            "html": html.HtmlLexer,
-            "javascript": javascript.JavascriptLexer,
-            "xml": html.XmlLexer,
-            "css": css.CssLexer,
-            "json": data.JsonLexer,
-            "yaml": data.YamlLexer,
-            "go": go.GoLexer,
-            "typescript": javascript.TypeScriptLexer,
-            "kotlin": jvm.KotlinLexer,
-            "php": php.PhpLexer,
-            "ruby": ruby.RubyLexer,
-            "rust": rust.RustLexer,
-            "lua": scripting.LuaLexer,
-            "java": jvm.JavaLexer,
-            "perl": perl.PerlLexer,
-            "swift": objective.SwiftLexer,
-            "react": jsx.JsxLexer,
-        }
         self.language = language
         self.check_lexer()
         self.configure_tags()
@@ -221,8 +165,8 @@ class CTkCodeBox(customtkinter.CTkTextbox):
         self.bind("<Double-Button-1>", self._on_double_click, add=True)
         self.bind("<Triple-Button-1>", self._on_triple_click, add=True)
 
-        if menu:
-            self.text_menu = TextMenu(self, fg_color=menu_fg_color, text_color=menu_text_color, hover_color=menu_hover_color)
+        if menu_settings.enabled:
+            self.text_menu = TextMenu(self, fg_color=menu_settings.fg_color, text_color=menu_settings.text_color, hover_color=menu_settings.hover_color)
 
     def check_lexer(self):
         """Resolve and set the lexer.
@@ -234,8 +178,8 @@ class CTkCodeBox(customtkinter.CTkTextbox):
             LanguageNotAvailableError: When string language is unknown.
         """
         if type(self.language) is str:
-            if self.language.lower() in self.common_langs:
-                self.lexer = self.common_langs[self.language.lower()]
+            if self.language.lower() in common_langs:
+                self.lexer = common_langs[self.language.lower()]
             else:
                 raise LanguageNotAvailableError("This language is not available, try to pass the pygments lexer instead. \nAvailable lexers: https://pygments.org/docs/lexers")
         else:
@@ -263,11 +207,8 @@ class CTkCodeBox(customtkinter.CTkTextbox):
 
     def _schedule_highlight(self, delay: int = 100):
         """Debounce highlighting to improve performance while typing."""
-        try:
-            if self._highlight_job is not None:
-                self.after_cancel(self._highlight_job)
-        except Exception:
-            pass
+        if getattr(self, "_highlight_job", None):
+            self.after_cancel(self._highlight_job)
         self._highlight_job = self.after(delay, self._apply_highlight)
 
     def _apply_highlight(self):
@@ -320,62 +261,113 @@ class CTkCodeBox(customtkinter.CTkTextbox):
             start_line = end_line
             start_index = end_index
 
-    def configure(self, param=None, **kwargs):
-        """Extended configure with CTkCodeBox options.
+    def _configure_type_check(self, arg, needType, argName):
+        def _check(needT, isList=False):
+            matching = False
+            if isinstance(arg, needT):
+                matching = True
+            else:
+                if not isList:
+                    raise ConfigureBadType(f'Type of provided arg "{argName}" should be {needT}, not {type(arg)}')
+            return matching
+        if isinstance(needType, list):
+            anyMatch = False
+            for t in needType:
+                anyMatch = _check(t, True)
+                if anyMatch:
+                    break
+            if not anyMatch:
+                raise ConfigureBadType(f'Type of provided arg "{argName}" should be one of next: {needType}, not {type(arg)}')
+            return anyMatch
+        else:
+            return _check(needType)
 
-        Supports additional keyword options: theme, language, wrap, history_enabled
-        select_color, cursor_color, history_max, history_cooldown, highlight_current_line,
-        current_line_color, indent_width.
+    def configure(self, param=None, **kwargs):  # param here ONLY for tklinenums module
+        """Extended configure with CTkCodeBox options.
         Re-highlights when theme or language changes.
         Remaining options are passed to the base widget.
+        Args:
+            **kwargs: Configuration parameters
+                    (Supports additional keyword options: theme, language, wrap_enabled, history_enabled,
+                    select_color, cursor_color, history_max, history_cooldown, highlight_current_line,
+                    current_line_color, history_settings, numbering_padx, numbering_auto_padx, indent_width)
+        Raises:
+            ConfigureBadType: If a type of provided kwargs not the right one
         """
-        if "theme" in kwargs:
+        one_action_handlers = {
+            "history_enabled": [self.set_history_enabled, bool],
+            "history_max": [self.set_history_limit, int],
+            "history_cooldown": [lambda val: setattr(self.history_settings, "cooldown", val), int],
+            "history_settings": [lambda val: setattr(self, "history_settings", val), HistorySettings],
+            "numbering_padx": [lambda val: self.line_nums.set_padx(val), int],
+            "numbering_auto_padx": [lambda val: setattr(self.line_nums, "auto_padx", val), bool],
+            "indent_width": [lambda val: setattr(self, "indent_width", val), int],
+            "wrap_enabled": [self.set_wrap, bool]
+        }
+        for param, value in list(kwargs.items()):
+            if (param in one_action_handlers and
+                    self._configure_type_check(value, one_action_handlers[param][1], param)):
+                one_action_handlers[param][0](value)
+                kwargs.pop(param)
+        if "theme" in kwargs and self._configure_type_check(kwargs["theme"], str, "theme"):
             self.theme_name = kwargs.pop("theme")
             self.configure_tags()
-        if "language" in kwargs:
+        if "language" in kwargs and self._configure_type_check(kwargs["language"], [str, lexer.Lexer], "language"):
             self.language = kwargs.pop("language")
             self.check_lexer()
-        if "wrap" in kwargs:
-            # Allow external wrap control via standard option
-            wrap_val = kwargs.get("wrap")
-            if isinstance(wrap_val, str):
-                self.wrap_enabled = (wrap_val != "none")
-            elif isinstance(wrap_val, bool):
-                self.wrap_enabled = wrap_val
-        if "select_color" in kwargs:
+        if "select_color" in kwargs and self._configure_type_check(kwargs["select_color"], str, "select_color"):
             self.select_color = kwargs.pop("select_color")
             self._textbox.config(selectbackground=self.select_color)
         else:
             # Keep default selection color updated if user hasn't provided one
             self._apply_selection_colors()
-        if "cursor_color" in kwargs:
+        if "cursor_color" in kwargs and self._configure_type_check(kwargs["cursor_color"], str, "cursor_color"):
             self.cursor_color = kwargs.pop("cursor_color")
             self._textbox.config(insertbackground=self.cursor_color)
-        if "history_enabled" in kwargs:
-            self.set_history_enabled(kwargs.pop("history_enabled"))
-        if "history_max" in kwargs:
-            self.set_history_limit(kwargs.pop("history_max"))
-        if "history_cooldown" in kwargs:
-            self._history_cooldown = kwargs.pop("history_cooldown")
-        if "highlight_current_line" in kwargs or "current_line_color" in kwargs:
+        if (("highlight_current_line" in kwargs and self._configure_type_check(kwargs["highlight_current_line"], bool, "highlight_current_line"))
+                or ("current_line_color" in kwargs and self._configure_type_check(kwargs["current_line_color"], str, "current_line_color"))
+                or ("highlight_current_line_color" in kwargs and self._configure_type_check(kwargs["highlight_current_line_color"], str, "highlight_current_line_color"))):
             self._highlight_current_line = kwargs.pop("highlight_current_line", self._highlight_current_line)
-            self._current_line_color = kwargs.pop("current_line_color", self._current_line_color)
+            self._current_line_color = kwargs.pop("current_line_color", kwargs.pop("highlight_current_line_color", self._current_line_color))
             self.tag_config("current_line", background=self._get_current_line_color())
-        if "indent_width" in kwargs:
-            self.indent_width = kwargs.pop("indent_width")
 
         # Re-apply highlighting if theme or language changed
         self._schedule_highlight(0)
-        super().configure(**kwargs)
+        if kwargs:
+            super().configure(**kwargs)
 
     def cget(self, param):
-        """Query CTkCodeBox options in addition to base options."""
-        if param == "theme":
-            return self.theme_name
-        elif getattr(self, param, None):
-            return getattr(self, param)
-        elif getattr(self, "_"+param, None):
-            return getattr(self, "_"+param)
+        """Get configuration parameter value with support for custom parameters.
+        Args:
+            param: Parameter name to retrieve
+        Returns:
+            Parameter value
+        """
+        custom_params = {
+            "theme": lambda: self.theme_name,
+            "wrap_enabled": lambda: self.wrap_enabled,
+            "edited": self.is_edited,
+            "undo_stack": lambda: self._undo_stack,
+            "redo_stack": lambda: self._redo_stack,
+            "history_cd_active": lambda: True if self._history_typing_cooldown else False,
+            "all_themes": lambda: self.all_themes,
+            "common_langs": lambda: common_langs,
+            "text_menu": lambda: self.text_menu,
+            "line_nums": lambda: self.line_nums,
+            "history_enabled": lambda: self.history_settings.enabled,
+            "history_max": lambda: self.history_settings.max,
+            "history_cooldown": lambda: self.history_settings.cooldown,
+            "history_settings": lambda: self.history_settings,
+            "numbering_auto_padx": lambda: self.line_nums.auto_padx,
+            "language": lambda: self.language,
+            "select_color": lambda: self.select_color,
+            "cursor_color": lambda: self.cursor_color,
+            "highlight_current_line": lambda: self._highlight_current_line,
+            "current_line_color": lambda: self._current_line_color,
+            "highlight_current_line_color": lambda: self._current_line_color,
+        }
+        if param in custom_params:
+            return custom_params[param]()
         return super().cget(param)
 
     # General helpers
@@ -449,7 +441,6 @@ class CTkCodeBox(customtkinter.CTkTextbox):
     # Indentation and auto-indent
     def _on_tab(self, event=None):
         """Indent selection or insert spaces at the caret.
-
         Returns:
              "break" to stop default Tab behavior.
         """
@@ -518,7 +509,6 @@ class CTkCodeBox(customtkinter.CTkTextbox):
 
     def _on_shift_tab(self, event=None):
         """Outdent selection or current line by one indent width.
-
         Returns:
             "break" to stop default Shift+Tab behavior.
         """
@@ -559,7 +549,10 @@ class CTkCodeBox(customtkinter.CTkTextbox):
             pass
 
     def _on_return(self, event=None):
-        """Insert newline and preserve leading whitespace for auto-indent."""
+        """Insert newline and preserve leading whitespace for auto-indent.
+        Returns:
+            "break" to stop default return behavior.
+        """
         try:
             # Auto-indent: copy leading whitespace from current line
             self._history_push_current()
@@ -590,24 +583,31 @@ class CTkCodeBox(customtkinter.CTkTextbox):
             pass
 
     def cut_text(self):
-        """Cut selected text to clipboard and notify change."""
+        """Cut selected text to clipboard and notify change.
+        Returns:
+            "Not found selected" when nothing is selected.
+            "Success" when success
+            Exception if something goes wrong
+        """
         try:
             if not self.tag_ranges("sel"):
                 return "Not found selected"
             self._history_push_current()
-            try:
-                text = self.get("sel.first", "sel.last")
-                pyperclip.copy(text)
-            except Exception:
-                pass
+            text = self.get("sel.first", "sel.last")
+            pyperclip.copy(text)
             self.delete("sel.first", "sel.last")
             self._notify_content_changed()
             return "Success"
         except Exception:
-            return None
+            return Exception
 
     def copy_text(self):
-        """Copy selected text to clipboard."""
+        """Copy selected text to clipboard.
+        Returns:
+            "Not found selected" when nothing is selected.
+            "Success" when success
+            Exception if something goes wrong
+        """
         try:
             if not self.tag_ranges("sel"):
                 return "Not found selected"
@@ -615,10 +615,15 @@ class CTkCodeBox(customtkinter.CTkTextbox):
             pyperclip.copy(text)
             return "Success"
         except Exception:
-            return None
+            return Exception
 
     def paste_text(self):
-        """Paste clipboard text, replacing selection if present, and refresh highlighting/lines."""
+        """Paste clipboard text, replacing selection if present, and refresh highlighting/lines.
+        Returns:
+            "Clipboard empty" when clipboard is empty.
+            "Success" when success
+            Exception if something goes wrong
+        """
         try:
             text = pyperclip.paste()
         except Exception:
@@ -637,53 +642,67 @@ class CTkCodeBox(customtkinter.CTkTextbox):
             self._schedule_highlight(0)
             return "Success"
         except Exception:
-            return None
+            return Exception
 
     def clear_all_text(self):
-        """Delete all content and notify change."""
+        """Delete all content and notify change.
+        Returns:
+            "Success" when success
+            Exception if something goes wrong
+        """
         try:
             self._history_push_current()
             self.delete("1.0", "end")
             self._notify_content_changed()
             return "Success"
         except Exception:
-            return None
+            return Exception
 
     def select_all_text(self):
-        """Select all content."""
+        """Select all content.
+        Returns:
+            "Success" when success
+            Exception if something goes wrong
+        """
         try:
             self.tag_add("sel", "1.0", "end-1c")
             return "Success"
         except Exception:
-            return None
+            return Exception
 
     # History API
     def set_history_enabled(self, enabled: bool):
         """Enable/disable the internal undo/redo history."""
-        self._history_enabled = bool(enabled)
-        return "Success"
+        self.history_settings.enabled = bool(enabled)
 
     def set_history_limit(self, limit: int):
-        """Set maximum number of undo frames to keep."""
+        """Set maximum number of undo frames to keep.
+        Returns:
+            "Success" when success
+            Exception if something goes wrong
+        """
         try:
-            self._history_max = max(0, int(limit))
+            self.history_settings.max = max(0, int(limit))
             # Trim if needed
-            if self._history_max and len(self._undo_stack) > self._history_max:
-                self._undo_stack = self._undo_stack[-self._history_max:]
+            if self.history_settings.max and len(self._undo_stack) > self.history_settings.max:
+                self._undo_stack = self._undo_stack[-self.history_settings.max:]
             return "Success"
         except Exception:
-            return None
+            return Exception
 
     def clear_history(self):
         """Clear undo and redo stacks."""
         self._undo_stack.clear()
         self._redo_stack.clear()
-        return "Success"
 
     def undo(self):
-        """Undo the last change if history is enabled."""
-
-        if not self._history_enabled or not self._undo_stack:
+        """Undo the last change if history is enabled.
+        Returns:
+            "Nothing to undo" when undo stack is empty or history is disabled.
+            "Success" when success
+            Exception if something goes wrong
+        """
+        if not self.history_settings.enabled or not self._undo_stack:
             return "Nothing to undo"
         try:
             current = self._save_state()
@@ -693,11 +712,16 @@ class CTkCodeBox(customtkinter.CTkTextbox):
             self._notify_content_changed()
             return "Success"
         except Exception:
-            return None
+            return Exception
 
     def redo(self):
-        """Redo the last undone change if available."""
-        if not self._history_enabled or not self._redo_stack:
+        """Redo the last undone change if available.
+        Returns:
+            "Nothing to redo" when redo stack is empty or history is disabled.
+            "Success" when success
+            Exception if something goes wrong
+        """
+        if not self.history_settings.enabled or not self._redo_stack:
             return "Nothing to redo"
         try:
             current = self._save_state()
@@ -707,7 +731,7 @@ class CTkCodeBox(customtkinter.CTkTextbox):
             self._notify_content_changed()
             return "Success"
         except Exception:
-            return None
+            return Exception
 
     # Internal history helpers
     def _save_state(self):
@@ -754,14 +778,14 @@ class CTkCodeBox(customtkinter.CTkTextbox):
 
     def _history_push_current(self):
         """Push a snapshot of the current buffer state onto the undo stack."""
-        if not self._history_enabled:
+        if not self.history_settings.enabled:
             return
         try:
             state = self._save_state()
             self._undo_stack.append(state)
             # Limit size
-            if self._history_max and len(self._undo_stack) > self._history_max:
-                self._undo_stack = self._undo_stack[-self._history_max:]
+            if self.history_settings.max and len(self._undo_stack) > self.history_settings.max:
+                self._undo_stack = self._undo_stack[-self.history_settings.max:]
             # Clear redo on new action
             self._redo_stack.clear()
         except Exception:
@@ -769,7 +793,7 @@ class CTkCodeBox(customtkinter.CTkTextbox):
 
     def _on_keypress_history(self, event):
         """Typing into undo frames and snapshot when replacing selection."""
-        if not self._history_enabled:
+        if not self.history_settings.enabled:
             return
         try:
             ks = getattr(event, 'keysym', '')
@@ -784,7 +808,7 @@ class CTkCodeBox(customtkinter.CTkTextbox):
                 return
             if self._history_typing_cooldown is None:
                 self._history_push_current()
-                self._history_typing_cooldown = self.after(self._history_cooldown, self._reset_history_cooldown)
+                self._history_typing_cooldown = self.after(self.history_settings.cooldown, self._reset_history_cooldown)
         except Exception:
             pass
 
@@ -879,7 +903,12 @@ class CTkCodeBox(customtkinter.CTkTextbox):
         return None
 
     def _on_backspace(self, event=None):
-        """Smart-backspace: delete paired quotes/brackets if cursor is between them."""
+        """Smart-backspace: delete paired quotes/brackets if cursor is between them.
+        Returns:
+            "break" to stop default backspace behavior.
+        """
+        self._history_push_current()
+        self._notify_content_changed()
         try:
             # If there is a selection, let default backspace handle it (history is handled elsewhere)
             if self.tag_ranges('sel'):
@@ -890,9 +919,7 @@ class CTkCodeBox(customtkinter.CTkTextbox):
             pairs = {('(', ')'), ('[', ']'), ('{', '}'), ('<', '>')}
             if prev_ch and next_ch:
                 if (prev_ch, next_ch) in pairs or (prev_ch == next_ch and prev_ch in ('"', "'", '`')):
-                    self._history_push_current()
                     self.delete("insert -1c", "insert +1c")
-                    self._notify_content_changed()
                     return "break"
         except Exception:
             pass
@@ -900,7 +927,10 @@ class CTkCodeBox(customtkinter.CTkTextbox):
 
     # Double-click smart selection (token expansion only)
     def _on_double_click(self, event):
-        """Expand selection to a token at clicked index (word/space/special)."""
+        """Expand selection to a token at clicked index (word/space/special).
+        Returns:
+            "break" to stop default double click behavior.
+        """
         try:
             idx = self.index(f"@{event.x},{event.y}")
             # Token selection
@@ -914,7 +944,10 @@ class CTkCodeBox(customtkinter.CTkTextbox):
         return None
 
     def _on_triple_click(self, event):
-        """Select the entire clicked line."""
+        """Select the entire clicked line.
+        Returns:
+            "break" to stop default triple click behavior.
+        """
         try:
             line_start = self.index(f"@{event.x},{event.y} linestart")
             line_end = self.index(f"@{event.x},{event.y} lineend")
